@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.8/apps/mkfbsdmnfst/mkfbsdmnfst.cc 10190 $
-// @(#) $Id: mkfbsdmnfst.cc 10190 2019-11-24 00:42:35Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.9.4/apps/mkfbsdmnfst/mkfbsdmnfst.cc 11639 $
+// @(#) $Id: mkfbsdmnfst.cc 11639 2021-01-04 03:44:40Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2016
 //  All rights reserved.
@@ -36,26 +36,9 @@
 
 //---------------------------------------------------------------------------
 //!  \file mkfbsdmnfst.cc
-//!  \brief NOT YET DOCUMENTED
+//!  \brief mkfbsdmnfst application code including main()
 //---------------------------------------------------------------------------
 
-//  I can set:
-//    name
-//    version
-//    origin
-//    comment
-//    desc
-//    arch
-//    www
-//    maintainer
-//    prefix
-//    files
-//
-//  I may need:
-//    conflict
-//    flatsize
-//    deps
-
 extern "C" {
   #include <fcntl.h>
   #include <fts.h>
@@ -68,6 +51,7 @@ extern "C" {
   #include <sqlite3.h>
 }
 #include <cstdlib>
+#include <filesystem>
 #include <fstream>
 #include <iomanip>
 #include <iostream>
@@ -78,23 +62,71 @@ extern "C" {
 #include <string>
 #include <vector>
 
+#include "DwmArguments.hh"
 #include "DwmFreeBSDPkgManifest.hh"
-#include "DwmOptArgs.hh"
 #include "DwmSvnTag.hh"
 
 using namespace std;
+namespace fs = std::filesystem;
+
 using Dwm::FreeBSDPkg::Manifest;
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.8/apps/mkfbsdmnfst/mkfbsdmnfst.cc 10190 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.9.4/apps/mkfbsdmnfst/mkfbsdmnfst.cc 11639 $");
 
-static Dwm::OptArgs  g_optargs;
+typedef   Dwm::Arguments<Dwm::Argument<'c',string>,
+                         Dwm::Argument<'d',string>,
+                         Dwm::Argument<'g',string>,
+                         Dwm::Argument<'m',string>,
+                         Dwm::Argument<'n',string>,
+                         Dwm::Argument<'o',string>,
+                         Dwm::Argument<'p',string>,
+                         Dwm::Argument<'r',string>,
+                         Dwm::Argument<'s',string,true>,
+                         Dwm::Argument<'u',string>,
+                         Dwm::Argument<'v',string>,
+                         Dwm::Argument<'w',string>> MyArgType;
+static MyArgType  g_args;
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void InitArgs()
+{
+  g_args.SetValueName<'c'>("comment");
+  g_args.SetHelp<'c'>("Set the comment ('comment:') value");
+  g_args.SetValueName<'d'>("desc");
+  g_args.SetHelp<'d'>("Set the description ('desc:') value");
+  g_args.SetValueName<'g'>("group");
+  g_args.Set<'g'>("wheel");
+  g_args.SetHelp<'g'>("Set the group ID of files (default is 'wheel')");
+  g_args.SetValueName<'m'>("maintainer");
+  g_args.SetHelp<'m'>("Set the maintainer's email address");
+  g_args.SetValueName<'n'>("name");
+  g_args.SetHelp<'n'>("Set the name of the package (e.g. 'libFooBar')");
+  g_args.SetValueName<'o'>("origin");
+  g_args.SetHelp<'o'>("Set the origin (e.g. 'devel/libFooBar')");
+  g_args.SetValueName<'p'>("prefix");
+  g_args.SetHelp<'p'>("Set the path where files will be installed");
+  g_args.SetValueName<'r'>("manifest");
+  g_args.SetHelp<'r'>("Read the given manifest file and ingest its settings");
+  g_args.SetValueName<'s'>("directory");
+  g_args.SetHelp<'s'>("Staging directory where files to be packaged are"
+                      " located");
+  g_args.Set<'u'>("root");
+  g_args.SetValueName<'u'>("user");
+  g_args.SetHelp<'u'>("Set the owner of files (default is 'root')");
+  g_args.SetValueName<'v'>("version");
+  g_args.SetHelp<'v'>("Set the version (e.g. '1.5.2')");
+  g_args.SetValueName<'w'>("URL");
+  g_args.SetHelp<'w'>("Set the software's official web site");
+}
 
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
 static void GetSharedLibs(const string & filename, set<string> & libs)
 {
-  string  lddcmd("ldd " + filename);
+  string  lddcmd("ldd " + filename + " 2>/dev/null");
   FILE    *lddpipe = popen(lddcmd.c_str(), "r");
   if (lddpipe) {
     regex   rgx(".+[ \\t]+[=][>][ \\t]+([^ \\t]+)[ \\t]+",
@@ -115,6 +147,30 @@ static void GetSharedLibs(const string & filename, set<string> & libs)
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static string GetInstalledPackageVersion(string packageName)
+{
+  string rc;
+  string  query("select packages.version from packages");
+  query += " where packages.name = '" + packageName + "'";
+  sqlite3  *ppdb;
+  if (sqlite3_open_v2("/var/db/pkg/local.sqlite", &ppdb,
+                      SQLITE_OPEN_READONLY, 0)
+      == SQLITE_OK) {
+    sqlite3_stmt *ppStmt;
+    if (sqlite3_prepare(ppdb, query.c_str(), -1, &ppStmt, 0) == SQLITE_OK) {
+      if (sqlite3_step(ppStmt) == SQLITE_ROW) {
+        rc = (const char *)sqlite3_column_text(ppStmt, 0);
+      }
+      sqlite3_finalize(ppStmt);
+    }
+    sqlite3_close_v2(ppdb);
+  }
+  return rc;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -255,7 +311,7 @@ vector<Manifest::File> GetManifestFiles(const string & dirName)
   vector<Manifest::File>  rc;
   vector<string>  filenames = GetFiles(dirName);
   for (auto f : filenames) {
-    Manifest::File  mf(f /*, GetSHA256(dirName + f) */);
+    Manifest::File  mf(f, GetSHA256(dirName + f));
     rc.push_back(mf);
   }
   return rc;
@@ -332,24 +388,88 @@ static bool HandleSpecialFile(const string & dirName, Manifest & manifest,
 }
 
 //----------------------------------------------------------------------------
-//!  Check for either mismatched package dependencies or missing dependencies
-//!  by looking for shared libraries in files in bin and sbin directories.
-//!  Patch up the dependencies if the version is mismatched, add them if
-//!  they're missing.
+//!  For any dependencies already in the manifest... if the version in
+//!  the manifest doesn't match the installed version, correct it.
 //----------------------------------------------------------------------------
-static void CheckPackageDependencies(const string & dirName,
-                                     Manifest & manifest)
+static void UpdatePackageDependencies(Manifest & manifest)
+{
+  for (auto it = manifest.Dependencies().begin();
+       it != manifest.Dependencies().end(); ++it) {
+    string  installedVersion =
+      GetInstalledPackageVersion(it->Name());
+    if ((! installedVersion.empty()) && (installedVersion != it->Version())) {
+      cerr << it->Name() << " version corrected from "
+           << it->Version() << " to " << installedVersion << '\n';
+      it->Version(installedVersion);
+    }
+  }
+}
+
+//----------------------------------------------------------------------------
+//!  Given the manifest and a vector of discovered installed dependencies,
+//!  make the manifest dependency entries match the installed dependencies
+//!  with respect to origin and version.  Also add any missing dependencies
+//!  to the manifest.
+//----------------------------------------------------------------------------
+static void
+CorrectDiscoveredDependencies(Manifest & manifest,
+                              const vector<Manifest::Dependency> & discDeps)
+{
+  for (auto dep : discDeps) {
+    auto  it =
+      find_if(manifest.Dependencies().begin(),
+              manifest.Dependencies().end(),
+              [&] (const Manifest::Dependency & mdep) 
+              {
+                return ((dep.Name() == mdep.Name())
+                        && ((dep.Version() != mdep.Version())
+                            || (dep.Origin() != mdep.Origin())));
+              });
+    if (it != manifest.Dependencies().end()) {
+      if (dep.Version() != it->Version()) {
+        cerr << "Dependency version mismatch, " << dep.Name()
+             << " version corrected to " << dep.Version() << '\n';
+      }
+      if (dep.Origin() != it->Origin()) {
+        cerr << "Dependency origin mismatch, " << dep.Name()
+             << " origin corrected to " << dep.Origin() << '\n';
+      }
+      *it = dep;
+    }
+    if (find_if(manifest.Dependencies().begin(),
+                manifest.Dependencies().end(),
+                [&] (const Manifest::Dependency & mdep) 
+                {
+                  return (dep.Name() == mdep.Name());
+                })
+        == manifest.Dependencies().end()) {
+      cerr << "Added dependency " << dep.Name()
+           << " version " << dep.Version() << '\n';
+      manifest.Dependencies().push_back(dep);
+    }
+  }
+  return;
+}
+
+//----------------------------------------------------------------------------
+//!  Check for either mismatched package dependencies or missing dependencies
+//!  by looking for shared libraries in files in the given directory.
+//!  Patch up the dependencies if the version or origin is mismatched, add
+//!  them if they're missing from the manifest.
+//----------------------------------------------------------------------------
+static void ScanForPackageDependencies(const string & dirName,
+                                       Manifest & manifest)
 {
   bool    rc = true;
-  regex   rgx("/bin/|/sbin/|/lib/lib.+\\.so\\.",
-              regex::ECMAScript|regex::optimize);
-  smatch  sm;
 
   set<string>  sharedLibs;
-  for (auto file : manifest.Files()) {
-    if (regex_search(file.Path(), sm, rgx)) {
-      string stagingPath = dirName + file.Path();
-      GetSharedLibs(stagingPath, sharedLibs);
+  cerr << "Scanning " << dirName << " for dependencies\n";
+  for (auto & p : fs::recursive_directory_iterator(dirName)) {
+    if (fs::is_regular_file(p.path())) {
+      if ((fs::status(p.path()).permissions() & fs::perms::owner_exec)
+          != fs::perms::none) {
+        GetSharedLibs(p.path().string(), sharedLibs);;
+      }
     }
   }
   if (! sharedLibs.empty()) {
@@ -359,44 +479,30 @@ static void CheckPackageDependencies(const string & dirName,
       vector<Manifest::Dependency>  dependencies;
       GetPackageInfo(packageDeps, dependencies);
       if (! dependencies.empty()) {
-        for (auto dep : dependencies) {
-          auto  it = find_if(manifest.Dependencies().begin(),
-                      manifest.Dependencies().end(),
-                      [&] (const Manifest::Dependency & mdep) 
-                      {
-                        return ((dep.Name() == mdep.Name())
-                                && ((dep.Version() != mdep.Version())
-                                    || (dep.Origin() != mdep.Origin())));
-                      });
-          if (it != manifest.Dependencies().end()) {
-            if (dep.Version() != it->Version()) {
-              cerr << "Dependency version mismatch, " << dep.Name()
-                   << " version corrected to " << dep.Version() << '\n';
-            }
-            if (dep.Origin() != it->Origin()) {
-              cerr << "Dependency origin mismatch, " << dep.Name()
-                   << " origin corrected to " << dep.Origin() << '\n';
-            }
-            *it = dep;
-          }
-          if (find_if(manifest.Dependencies().begin(),
-                      manifest.Dependencies().end(),
-                      [&] (const Manifest::Dependency & mdep) 
-                      {
-                        return (dep.Name() == mdep.Name());
-                      })
-              == manifest.Dependencies().end()) {
-            cerr << "Added dependency " << dep.Name()
-                 << " version " << dep.Version() << '\n';
-            manifest.Dependencies().push_back(dep);
-          }
-        }
+        CorrectDiscoveredDependencies(manifest, dependencies);
       }
     }
   }
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static map<char,string> ManifestFieldArgs()
+{
+  map<char,string>  rc;
+  if (! g_args.Get<'c'>().empty())  {   rc['c'] = g_args.Get<'c'>();  }
+  if (! g_args.Get<'d'>().empty())  {   rc['d'] = g_args.Get<'d'>();  }
+  if (! g_args.Get<'m'>().empty())  {   rc['m'] = g_args.Get<'m'>();  }
+  if (! g_args.Get<'n'>().empty())  {   rc['n'] = g_args.Get<'n'>();  }
+  if (! g_args.Get<'o'>().empty())  {   rc['o'] = g_args.Get<'o'>();  }
+  if (! g_args.Get<'p'>().empty())  {   rc['p'] = g_args.Get<'p'>();  }
+  if (! g_args.Get<'v'>().empty())  {   rc['v'] = g_args.Get<'v'>();  }
+  if (! g_args.Get<'w'>().empty())  {   rc['w'] = g_args.Get<'w'>();  }
+  return rc;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -421,11 +527,15 @@ bool PopulateManifest(const string & dirName, Manifest & manifest)
   bool  rc = false;
   vector<Manifest::File>  manifestFiles = GetManifestFiles(dirName);
   if (! manifestFiles.empty()) {
-    for (auto it : fieldSetters) {
-      if (! g_optargs.Get<string>(it.first).empty()) {
-        //  Weird syntax is due to calling a pointer to member function.
-        //  (manifest.*(it.second)) is the pointer to member function.
-        (manifest.*(it.second))(g_optargs.Get<string>(it.first));
+    map<char,string>  mnfstFieldArgs = ManifestFieldArgs();
+    if (! mnfstFieldArgs.empty()) {
+      for (auto mnfstField : mnfstFieldArgs) {
+        auto  it = fieldSetters.find(mnfstField.first);
+        if (it != fieldSetters.end()) {
+          //  Weird syntax is due to calling a pointer to member function.
+          //  (manifest.*(it->second)) is the pointer to member function.
+          (manifest.*(it->second))(mnfstField.second);
+        }
       }
     }
     set<string> sharedLibDeps;
@@ -437,8 +547,8 @@ bool PopulateManifest(const string & dirName, Manifest & manifest)
                   { return item.Path() == mfit.Path(); })
           == manifest.Files().end()) {
         if (! HandleSpecialFile(dirName, manifest, mfit)) {
-          mfit.Group(g_optargs.Get<string>('g'));
-          mfit.User(g_optargs.Get<string>('u'));
+          mfit.Group(g_args.Get<'g'>());
+          mfit.User(g_args.Get<'u'>());
           manifest.Files().push_back(mfit);
         }
       }
@@ -457,47 +567,36 @@ bool PopulateManifest(const string & dirName, Manifest & manifest)
 //----------------------------------------------------------------------------
 int main(int argc, char *argv[])
 {
-  g_optargs.AddOptArg("n:", "name", false, "",
-                      "name of package (e.g. 'libDwm')");
-  g_optargs.AddOptArg("v:", "version", false, "", "version (e.g. '1.5.2')");
-  g_optargs.AddOptArg("o:", "origin", false, "",
-                      "origin (e.g. 'devel/libDwm')");
-  g_optargs.AddOptArg("c:", "comment", false, "", "comment");
-  g_optargs.AddOptArg("d:", "desc", false, "", "description");
-  g_optargs.AddOptArg("g:", "group", false, "wheel",
-                      "group ID of files (default is 'wheel')");
-  g_optargs.AddOptArg("w:", "www", false, "", "software's official web site");
-  g_optargs.AddOptArg("m:", "maintainer", false, "",
-                      "maintainer's email address");
-  g_optargs.AddOptArg("p:", "prefix", false, "",
-                      "path where files will be installed");
-  g_optargs.AddOptArg("r:", "read", false, "", "read manifest file first");
-  g_optargs.AddOptArg("u:", "user", false, "root",
-                      "owner of files (default is 'root')");
-  g_optargs.AddNormalArg("directory", true);
-  
-  int nextArg = g_optargs.Parse(argc, argv);
-  if (nextArg > (argc - 1)) {
-    g_optargs.Usage(argv[0]);
-    return 1;
+  InitArgs();
+  int  argind = g_args.Parse(argc, argv);
+  if (argind < 0) {
+    cerr << g_args.Usage(argv[0], "[dependency_scan_path(s)...]");
+    exit(1);
   }
-
+  
   Dwm::FreeBSDPkg::Manifest  manifest;
 
-  if (! g_optargs.Get<string>('r').empty()) {
-    manifest.Parse(g_optargs.Get<string>('r').c_str());
+  if (! g_args.Get<'r'>().empty()) {
+    manifest.Parse(g_args.Get<'r'>().c_str());
   }
   
   struct stat  statbuf;
-  if (stat(argv[nextArg], &statbuf) == 0) {
+  if (stat(g_args.Get<'s'>().c_str(), &statbuf) == 0) {
     if (statbuf.st_mode & S_IFDIR) {
       //  Add files from directory argv[nextArg] to the manifest.
-      if (PopulateManifest(argv[nextArg], manifest)) {
-        //  check for missing/mismatched dependencies.
-        CheckPackageDependencies(argv[nextArg], manifest);
+      if (PopulateManifest(g_args.Get<'s'>(), manifest)) {
+        //  Update any dependencies that were already in the manifest, to
+        //  match the installed version of the dependency.
+        UpdatePackageDependencies(manifest);
+        //  Scan for missing/mismatched dependencies in staging directory.
+        ScanForPackageDependencies(g_args.Get<'s'>(), manifest);
+        //  And then in any other directories given on the command line.
+        for ( ; argind < argc; ++argind) {
+          ScanForPackageDependencies(argv[argind], manifest);
+        }
         //  Check for missing files.
         vector<Manifest::File>  missingFiles =
-          manifest.MissingFiles(argv[nextArg]);
+          manifest.MissingFiles(g_args.Get<'s'>());
         if (missingFiles.empty()) {
           //  No missing files.  Emit the manifest.
           cout << manifest;
@@ -513,13 +612,13 @@ int main(int argc, char *argv[])
       }
     }
     else {
-      cerr << argv[nextArg] << " is not a directory!\n";
+      cerr << g_args.Get<'s'>() << " is not a directory!\n";
       return 1;
     }
   }
   else {
-    cerr << "Failed to stat " << argv[nextArg] << ": " << strerror(errno)
-         << '\n';
+    cerr << "Failed to stat " << g_args.Get<'s'>()
+         << ": " << strerror(errno) << '\n';
     return 1;
   }
 
