@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath$
-// @(#) $Id$
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.9.4/src/DwmIpAddress.cc 11463 $
+// @(#) $Id: DwmIpAddress.cc 11463 2020-12-16 02:33:30Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2020
 //  All rights reserved.
@@ -51,6 +51,9 @@ extern "C" {
 #include "DwmGZIO.hh"
 #include "DwmIOUtils.hh"
 #include "DwmStreamIO.hh"
+#include "DwmSvnTag.hh"
+
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.9.4/src/DwmIpAddress.cc 11463 $");
 
 namespace Dwm {
 
