@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath$
-// @(#) $Id$
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.9.4/src/DwmIpPrefix.cc 11464 $
+// @(#) $Id: DwmIpPrefix.cc 11464 2020-12-16 02:34:19Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2020
 //  All rights reserved.
@@ -49,7 +49,7 @@
 #include "DwmStreamIO.hh"
 #include "DwmSvnTag.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath:$");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.9.4/src/DwmIpPrefix.cc 11464 $");
 
 namespace Dwm {
 
